# =================================================================
# Licensed Materials - Property of IBM
#
# (c) Copyright IBM Corp. 2020 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# =================================================================
# IBM starts - 77754
import os
import re
import time

from cloudinit.settings import PER_ALWAYS
from cloudinit import util
from cloudinit import netinfo

frequency = PER_ALWAYS

RMCCTRL = '/usr/sbin/rsct/bin/rmcctrl'

def handle(name, _cfg, _cloud, log, _args):
    info = netinfo.netdev_info()
    # IBM starts - RTC 77854
    # COLO RMC issue
    # Setting NM_CONTROLLED=no for IPv6 interface
    # making it down and up
    info_len = len(info.keys())

    # IBM start - RTC 74253, 74635 and 74668
    log.debug('Making the ip up')
    # Trying to make the interfaces up
    for info_count in range(info_len):
        try:
            if 'lo' in list(info.keys())[info_count]:
                    log.debug('Skipping localhost interface')
                    continue
            interface = list(info.keys())[info_count]
            # RTC - 77819 and 80505
            # Reording the commands to be run
            # Now, refresh of IPv6 is done after setting NM_CONTROLLED = no
            # on IPv6 interface
            if not info[str(list(info.keys())[info_count])]['ipv4']:
                # As IPv6 was fluctuating we removed IPV^ details and restarted
                # Network Manager
                disable_NM_ipv6(interface)
                # Refreshing the ipv6 since we don't want it to be handled by
                # NetworkManager
                refresh_ipv6(interface)
        except:
            util.logexc(log, 'Interface is not up')
            pass

    refresh_rmc()

def refresh_ipv6(interface):
    util.subp(['ip', 'link', 'set', interface, 'down'])
    util.subp(['ip', 'link', 'set', interface, 'up'])


def disable_NM_ipv6(interface):
    interface_file = '/etc/sysconfig/network-scripts/ifcfg-' + interface
    pattern = re.compile("NM_CONTROLLED")
    NM_CONTROL = False
    with open(interface_file) as f:
        data = f.read()
        if 'NM_CONTROLLED' in data:
            NM_CONTROL = True
        if 'IPV6ADDR' in data:
            if NM_CONTROL:
                util.subp(['sed', '-i', 's/NM_CONTROLLED=yes/NM_CONTROLLED=no/g', interface_file])
            else:
                with open(interface_file, "a") as file_object:
                    file_object.write("NM_CONTROLLED=no")
            util.subp(['sed', '-i', '/^IPV6INIT/d', interface_file])
            util.subp(['sed', '-i', '/^IPV6ADDR/d', interface_file])
            util.subp(['sed', '-i', '/^IPADDR6/d', interface_file])
            util.subp(['systemctl', 'restart', 'NetworkManager'])


def refresh_rmc():
    try:
        util.subp([RMCCTRL, '-z'])
        util.subp([RMCCTRL, '-s'])
    except Exception as e:
        util.logexc(log, 'Failed to refresh the RMC subsystem.')
        raise
# IBM ends
